import 'dart:developer';

import 'package:hub/features/module/data/data_sources/index.dart';
import 'package:hub/features/module/domain/index.dart';
import 'package:hub/features/storage/index.dart';
import 'package:sqflite/sqflite.dart';

typedef LicenseStatus = bool;

class LicenseRepositoryImpl implements LicenseRepository {
  final LicenseLocalDataSource localDataSource = LicenseLocalDataSourceImpl();
  final LicenseRemoteDataSource remoteDataSource =
      LicenseRemoteDataSourceImpl();
  final Database database = DatabaseService.database;

  LicenseRepositoryImpl();

  @override
  Future<bool> updateLicense() async {
    log('updateLicense');
    bool result = false;
    final bool isNewVersion = await localDataSource.isNewVersion();
    result = await remoteDataSource.setupLicense(database, isNewVersion);
    if (isNewVersion) {
      result = await remoteDataSource.fetchLicenses(isNewVersion);
    }

    return result;
  }

  @override
  Future<void> cleanLicense() async {
    return await remoteDataSource.cleanLicense();
  }

  @override
  Future<String?> getLicense(String key) async {
    final response = await localDataSource.get(key);
    return response;
  }

  @override
  Future<void> setLicense<T>(String key, T value) async {
    return await localDataSource.set(key, value);
  }
}
